% 'Matchine' software by Joan Alexis Glaunes, Copyright (C) Universite Paris Descartes, All Rights Reserved 2005-2006, version 1.0
%
% Any use of this code should make reference to at least one of the following references:
%
% - [for the use of 'measure matching']:
% J. Glaunes, L. Younes and A. Trouve, Diffeomorphic matching of distributions: A new approach for unlabelled point-sets and sub-manifolds matching, Proc. of the 2004 IEEE Computer Society Conference on Computer Vision and Pattern Recognition. (CVPR'04), Vol. 2, pp. 712--718, DOI: 10.1109/CVPR.2004.81
%
% - [for the use of 'curve matching']:
% J. Glaunes, A. Qiu, M. Miller, L. Younes, Large Deformation Diffeomorphic Metric Curve Mapping, International Journal of Computer vision, Springer, 2008, Vol. 80, No. 3, pp. 317--336, DOI: 10.1007/s11263-008-0141-9
%
% - [for the use of 'surface matching']:
% M. Vaillant and J. Glaunes, Surface Matching via Currents, Proc. of Information Processing in Medical Imaging (IPMI'05), Lecture Notes in Computer Science vol. 3565, Springer 2005, pp. 381--392


function builtinker(kernel,dkernel,flag)

% build specified kernels for matlab codes in current directory
% kernel and dkernel are strings containing the kernel and its derivative
% with argument argin and expressed as functions of -|x-y|^2/sigma^2
% eg1: builtinker('exp(argin)','exp(argin)')
% eg2: builtinker('1/(1-argin)','1/(1-argin)^2')
% eg3: builtinker('sobolev3(3,argin)','dsobolev3(3,argin)')
% optional argument flag is to specify which kernels should be changed (default is all):
% 'V' = deformation kernel,
% 'I' = measure matching kernel,
% 'W' = currents matching kernel
% eg4: builtinker('exp(argin)','exp(argin)','V')
% eg5: builtinker('exp(argin)','exp(argin)',{'I','W'})
%
% contact : Joan Glauns - http://www.mi.parisdescartes.fr/~glaunes/

switch nargin
    case 2
        flaglist = {'V','I','W'};
    case 3
        if iscell(flag)
            flaglist = flag;
        else
            flaglist{1} = flag;
        end
end

for k = 1:length(flaglist)
    listker{k} = ['ker',flaglist{k}];
    listder{k} = ['der',flaglist{k}];
end

nomodif = 1;

sourcetab = dir('*.m');
for s = 1:length(sourcetab);
    source = sourcetab(s).name;
    if ~strcmp(source,'builtinker.m')
        f = fopen(source,'r');
        code = fscanf(f,'%c',inf);
        fclose(f);
        if findstr(code,'%% BUILT IN KERNEL')
            for k = 1:length(listker)
                code = regexprep(code,['[^\n\r]*%% BUILT IN KERNEL FLAG ',listker{k}], ...
                    [listker{k},' = ''',kernel,''';  %% BUILT IN KERNEL FLAG ',listker{k}]);
                code = regexprep(code,['[^\n\r]*%% BUILT IN KERNEL ',listker{k}], ...
                    ['argout = ',kernel,';  %% BUILT IN KERNEL ',listker{k}]);
                code = regexprep(code,['[^\n\r]*%% BUILT IN KERNEL ',listder{k}], ...
                    ['argout = ',dkernel,';  %% BUILT IN KERNEL ',listder{k}]);
            end
            f = fopen(source,'w');
            fprintf(f,'%c',code);
            fclose(f);
            disp(['file ',source,' modified'])
            nomodif = 0;
        end
    end
end

if nomodif
    disp('Nothing to do in this directory')
end
